/* LC-meter commands
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <dos.h>
#include <conio.h>
#include "global.h"
#include "vars.h"
#include "lcm.h"
#include "freqcnt.h"

#define	GATELCM		GATE500MS
#define	DELAYLCM	50
#define	MAXVALS		10

/* Read and display ports
 */
int
doshowports(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned char p0,p1,p2;

	if (argc != 1) {
		fprintf(stderr,"Usage: showports\n");
		return -1;
	}
	p0 = inp(lpt_base);
	p1 = inp(lpt_base+1);
	p2 = inp(lpt_base+2);
	printf("base:0x%02x base+1:0x%02x base+2:0x%02x\n",p0, p1, p2);
	return(0);
}

/* Relays are connected to LPT port D0-2
 * D0: Reference C
 * D1: Measure L
 * D2: Measure C
 *
 * Parameter is hex number 0 - 7
 */
int
dorelays(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	unsigned char bits;

	if (argc != 2) {
		fprintf(stderr,"Usage: relays 0-7\n");
		return -1;
	}
	bits = htoi(argv[1]);
	outp(lpt_base,bits);
	return(0);
}

/* Set LPT port
 * TO BE IMPLEMENTED
 */
int
dolpt(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return(0);
}

/* Measure frequency on pin 10 (~ACK) of lpt port
 */
int
dofreq(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int	r;		/* result variable	*/
	double f;		/* frequency		*/

	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
	} else {
		f = fcount_read();
		printf("%g\n",f);
	}
	return(0);
}

/* Calibrate
 */
int
docal(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int r;

	r = calibrate();
	if (r >= 0 && debug > 0) {
		printf("Cref = ");print_c(cref);printf("\n");
		printf("C0   = ");print_c(c0);printf("\n");
		printf("L0   = ");print_l(l0);printf("\n");
	}

	return r;
}

int
calibrate(void)
{
	int	r;			/* result variable	*/
	double f0,f1;			/* frequencies		*/
	double l,c;			/* Calculated L0 end C0	*/
	double dl,dc;			/* delta l and delta c	*/

	/* All relays off */
	if (debug > 0)
		printf("OUT 0x%x\n",RELAY_NONE);
	outp(lpt_base,RELAY_NONE);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return(-1);
	} else {
		f0 = fcount_read();
	}

	/* Switch in Cref */
	if (debug > 0)
		printf("OUT 0x%x\n",RELAY_CREF);
	outp(lpt_base,RELAY_CREF);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return(-1);
	} else {
		f1 = fcount_read();
	}
	if (debug > 0) {
		printf("F0 = ");print_f(f0);printf("\n");
		printf("F1 = ");print_f(f1);printf("\n");
	}

	/* All relays off */
	if (debug > 0)
		printf("OUT 0x%x\n",RELAY_NONE);
	outp(lpt_base,RELAY_NONE);

	/* Calculate
	 * f0 = 1 / ( 2 * PI * SQRT(L0 * C0))
	 * f1 = 1 / ( 2 * PI * SQRT(L0 * (C0 + Cref)))
	 *
	 * == >
	 *
	 * C0 = Cref * f1 * f1 / (f0 * f0 - f1 * f1)
	 * L0 = 1 / ( 4 * PI * PI * f0 * f0 * C0)
	 */
	c = cref * f1 * f1 / ( f0 * f0 - f1 * f1);
	l = 1 / (4 * M_PI * M_PI * f0 * f0 * c);

	/* Save values */
	if (debug > 0) {
		dc = (c - c0) / c0;
		printf("C0: %g -> %g (%.1g%%)\n",c0,c,dc*100.0);
		dl = (l - l0) / l0;
		printf("L0: %g -> %g (%.1g%%)\n",l0,l,dl*100.0);
	}
	c0 = c;
	l0 = l;

	return(0);
}
 
/* Measure Cx
 * If called with 'cx0' sets cx0
 */
int
docx(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int r;				/* Result variable	*/
	double c;			/* Cx			*/
	int n;				/* Repeat counter	*/
	double cvals[MAXVALS];		/* Result array		*/
	int i;
	int zflag;			/* Signals Cx0 measurement*/
	double cav,cmin,cmax;		/* Minimum / maximum	*/

	/* Cx or Cx0 ?*/
	if (strcmp(argv[0],"cx") == 0)
		zflag = 0;
	else
		zflag = 1;

	/* Argument ? */
	if (argc == 1)
		n = 1;
	else
		n = atoi(argv[1]);

	/* Bound check */
	if (n <=0 ) {
		n = 1;
		printf("Repeat count set to 1\n");
	} else if (n > MAXVALS) {
		n = MAXVALS;
		printf("Repeat count set to %d\n",MAXVALS);
	}

	/* Measurements */
	for(i=0 ; i < n ; i++) {
		/* Calibrate */
		r = calibrate();

		/* Measure */
		c = measure_c();
		if (zflag == 0)
			c -= cx0;
		printf("%2d: Cx = ",i+1);print_c(c);printf("\n");

		/* Update array */
		cvals[i] = c;
	}

	/* Statistics */
	cmin = cvals[0];
	cmax = cvals[0];
	for(i = 1 ; i < n ; i++) {
		if (cvals[i] < cmin)
			cmin = cvals[i];
		if (cvals[i] > cmax)
			cmax = cvals[i];
	}
	cav = (cmax + cmin) / 2;
	printf("\n");
	printf("Cav  = ");print_c(cav);printf("\n");
	printf("Cmin = ");print_c(cmin);
	printf("(%.2g%%)\n",(cmin-cav)*100.0/cav);
	printf("Cmax = ");print_c(cmax);
	printf("(%.2g%%)\n",(cmax-cav)*100.0/cav);

	/* Save cx0 if asked */
	if (zflag)
		cx0 = cav;

	/* All done */
	return(0);
}

double
measure_c(void)
{
	int	r;			/* result variable	*/
	double f0,f1;			/* frequencies		*/
	double c;			/* Calculated L0 end C0	*/

	/* All relays off */
	outp(lpt_base,RELAY_NONE);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return( -1.0);
	} else {
		f0 = fcount_read();
	}

	/* Switch in Cx */
	outp(lpt_base,RELAY_CX);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return(-1.0);
	} else {
		f1 = fcount_read();
	}

	/* All relays off */
	outp(lpt_base,RELAY_NONE);

	/* Sanity check */
	if (debug > 0) {
		printf("F0 = ");print_f(f0);printf("\n");
		printf("F1 = ");print_f(f1);printf("\n");
	}
	if (f1 > f0) {
		printf("Not a capacitance\n");
		return -1.0;
	}

	/* Calculate
	 * f0 = 1 / ( 2 * PI * SQRT(L0 * C0))
	 * f1 = 1 / ( 2 * PI * SQRT(L0 * (C0 + Cx)))
	 *
	 * == >
	 *
	 * Cx = C0 * (f0 * f0 - f1 * f1)/(f1 * f1)
	 */
	c = c0 * (f0 * f0 - f1 * f1) / (f1 * f1);

	return(c);
}

/* Measure LCx
 */
int
dolcx(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int r;				/* Result variable	*/
	double c,l,fres;		/* Cx, Lx, Fres		*/
	int n;				/* Repeat counter	*/
	double cvals[MAXVALS];		/* Result arrays	*/
	double lvals[MAXVALS];
	int i;
	double cav,cmin,cmax;		/* Minimum / maximum C	*/
	double lav,lmin,lmax;		/* Minimum / maximum L	*/

	/* Argument ? */
	if (argc == 1)
		n = 1;
	else
		n = atoi(argv[1]);

	/* Bound check */
	if (n <=0 ) {
		n = 1;
		printf("Repeat count set to 1\n");
	} else if (n > MAXVALS) {
		n = MAXVALS;
		printf("Repeat count set to %d\n",MAXVALS);
	}

	/* Measurements */
	for(i=0 ; i < n ; i++) {
		/* Calibrate */
		r = calibrate();

		/* Measure */
		measure_lc(&c,&l);
		fres = 1 / (2 * M_PI * sqrt(c * l));
		printf("%2d: Cx = ",i+1);print_c(c);
		printf(" Lx = ");print_l(l);
		printf(" Fres = ");print_f(fres);printf("\n");

		/* Update array */
		cvals[i] = c;
		lvals[i] = l;
	}

	/* Statistics */
	cmin = cvals[0];
	cmax = cvals[0];
	for(i = 1 ; i < n ; i++) {
		if (cvals[i] < cmin)
			cmin = cvals[i];
		if (cvals[i] > cmax)
			cmax = cvals[i];
	}
	cav = (cmax + cmin) / 2;
	printf("\n");
	printf("Cav  = ");print_c(cav);printf("\n");
	printf("Cmin = ");print_c(cmin);
	printf("(%.2g%%)\n",(cmin-cav)*100.0/cav);
	printf("Cmax = ");print_c(cmax);
	printf("(%.2g%%)\n",(cmax-cav)*100.0/cav);

	lmin = lvals[0];
	lmax = lvals[0];
	for(i = 1 ; i < n ; i++) {
		if (lvals[i] < lmin)
			lmin = lvals[i];
		if (lvals[i] > lmax)
			lmax = lvals[i];
	}
	lav = (lmax + lmin) / 2;
	printf("\n");
	printf("Lav  = ");print_l(lav);printf("\n");
	printf("Lmin = ");print_l(lmin);
	printf("(%.2g%%)\n",(lmin-lav)*100.0/lav);
	printf("Lmax = ");print_l(lmax);
	printf("(%.2g%%)\n",(lmax-lav)*100.0/lav);

	/* All done */
	return(0);
}

void
measure_lc(double *c, double *l)
{
	int	r;			/* result variable	*/
	double f0,f1;			/* frequencies		*/
	double ct;			/* Cx + C0		*/

	/* Switch in LCx */
	outp(lpt_base,RELAY_CX|RELAY_LX);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		*c = *l = -1.0;
		return;
	} else {
		f0 = fcount_read();
	}

	/* Switch in LCx and Cref */
	outp(lpt_base,RELAY_CREF|RELAY_CX|RELAY_LX);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		*c = *l = -1.0;
		return;
	} else {
		f1 = fcount_read();
	}

	/* All relays off */
	outp(lpt_base,RELAY_NONE);

	/* Sanity check */
	if (debug > 0) {
		printf("F0 = ");print_f(f0);printf("\n");
		printf("F1 = ");print_f(f1);printf("\n");
	}
	if (f1 > f0) {
		printf("F1 > F0\n");
		printf("F0 = ");print_f(f0);printf("\n");
		printf("F1 = ");print_f(f1);printf("\n");
		*c = *l = -1.0;
		return;
	}

	/* Calculate
	 * Ct = C0 + Cx
	 * f0 = 1 / ( 2 * PI * SQRT(Lx * Ct))
	 * f1 = 1 / ( 2 * PI * SQRT(Lx * (Ct + Cref)))
	 *
	 * == >
	 *
	 * Ct = Cref * f1 * f1 / (f0 * f0 - f1 * f1)
	 * Lx = 1 / ( 4 * PI * PI * f0 * f0 * Ct)
	 */
	ct = cref * f1 * f1 / ( f0 * f0 - f1 * f1);
	*l = 1 / (4 * M_PI * M_PI * f0 * f0 * ct);

	*c = ct - c0;
	return;
}
 
/* Calibrate
 */
int
dolx(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int r;				/* Result variable	*/
	double l;			/* Lx			*/
	int n;				/* Repeat counter	*/
	double lvals[MAXVALS];		/* Result array		*/
	int i;
	int zflag;			/* Signals Lx0 measurement*/
	double lav,lmin,lmax;		/* Average / Minimum / maximum	*/

	/* Lx or Lx0 ?*/
	if (strcmp(argv[0],"lx") == 0)
		zflag = 0;
	else
		zflag = 1;

	/* Argument ? */
	if (argc == 1)
		n = 1;
	else
		n = atoi(argv[1]);

	/* Bound check */
	if (n <=0 ) {
		n = 1;
		printf("Repeat count set to 1\n");
	} else if (n > MAXVALS) {
		n = MAXVALS;
		printf("Repeat count set to %d\n",MAXVALS);
	}

	/* Measurements */
	for(i=0 ; i < n ; i++) {
		/* Calibrate */
		if (calibrate() < 0)
			return -1;

		/* Measure */
		if ((l = measure_l()) < 0.0)
			return -1;
		if (zflag == 0)
			l -= lx0;

		printf("%2d: Lx = ",i+1);print_l(l);printf("\n");

		/* Update array */
		lvals[i] = l;
	}

	/* Statistics */
	lmin = 1.0;
	lmax = 1.0e-15;
	for(i = 0 ; i < n ; i++) {
		if (lvals[i] < lmin)
			lmin = lvals[i];
		if (lvals[i] > lmax)
			lmax = lvals[i];
	}
	lav = (lmax + lmin) / 2;
	printf("\n");
	printf("Lav  = ");print_l(lav);printf("\n");
	printf("Lmin = ");print_l(lmin);
	printf("(%.2g%%)\n",(lmin-lav)*100.0/lav);
	printf("Lmax = ");print_l(lmax);
	printf("(%.2g%%)\n",(lmax-lav)*100.0/lav);

	/* Save Lx0 if asked */
	if (zflag)
		lx0 = lav;

	/* All done */
	return(0);
}

double
measure_l(void)
{
	int	r;			/* result variable	*/
	double f0,f1;			/* frequencies		*/
	double l;			/* Calculated L0 end C0	*/

	/* All relays off */
	outp(lpt_base,RELAY_NONE);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return(-1.0);
	} else {
		f0 = fcount_read();
	}

	/* Switch in Cref */
	outp(lpt_base,RELAY_LX);

	/* Let oscillator stabilize */
	delay(DELAYLCM);

	/* Measure */
	fcount_start(GATELCM);	/* gate = 1 sec		*/
	r = fcount_wait();	/* wait for results	*/
	if (r < 0) {
		printf("Low frequency\n");
		return(-1.0);
	} else {
		f1 = fcount_read();
	}

	/* All relays off */
	outp(lpt_base,RELAY_NONE);

	/* Sanity check */
	if (debug > 0) {
		printf("F0 = ");print_f(f0);printf("\n");
		printf("F1 = ");print_f(f1);printf("\n");
	}
	if (f1 > f0) {
		printf("Not an inductance\n");
		return -1.0;
	}
	/* Calculate
	 * f0 = 1 / ( 2 * PI * SQRT(L0 * C0))
	 * f1 = 1 / ( 2 * PI * SQRT((L0 + Lx) * C0))
	 *
	 * == >
	 *
	 * Lx = L0 * (f0 * f0 - f1 * f1) / (f1 * f1)
	 */
	l = l0 * (f0 * f0 - f1 * f1) / (f1 * f1);

	return(l);
}
 
/* Utility routines */

/* Print capacitance
 */
void
print_c(double c)
{
	if (c < 1e-9)
		printf("%g pF",c*1e12);
	else if (c < 1e-6)
		printf("%g nF",c*1e9);
	else if (c < 1e-3)
		printf("%g uF",c*1e6);
	else
		printf("%g F",c);
}

/* Print inductance
 */
void
print_l(double l)
{
	if (l < 1e-6)
		printf("%g nH",l*1e9);
	else if (l < 1e-3)
		printf("%g uH",l*1e6);
	else if (l < 1.0)
		printf("%g mH",l*1e3);
	else
		printf("%g H",l);
	
}

/* Print frequency
 */
void
print_f(double f)
{
	if (f > 1e9)
		printf("%g GHz",f/1e9);
	else if (f > 1e6)
		printf("%g MHz",f/1e6);
	else if (f > 1e3)
		printf("%g KHz",f/1e3);
	else
		printf("%g Hz",f);
}
