/*  Variable set/display routines
 *
 *  (c) Copyright 1996,1997 Jakob Schripsema , PA3HDF  All Rights Reserved
 *  Permission for non-commercial use is hereby granted provided this notice
 *  is retained.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "global.h"
#include "cmdparse.h"
#include "vars.h"
#include "lcm.h"

/*
 * Global variables
 */
double	cref = 1.37e-9;			/* Reference capacitor Cref	*/
double	l0 = 100e-6;			/* L0 nominal value 100 uH	*/
double	c0 = 680e-12;			/* C0 nominal value 680 pF	*/
double	cx0 = 0.0;			/* Parasitic Cx, open inpu)	*/
double	lx0 = 0.0;			/* Parasitic Lx, shortened input*/
int	lpt_base = 0x378;		/* LPT port spec.		*/
int	lpt_irq = 7;
int	debug = 0;			/* Debug flag			*/

#define	T_NONE		0
#define	T_INT		1
#define	T_LONG		2
#define	T_DOUBLE	3
#define	T_ON_OFF	4
#define	T_STRING	5

struct var_desc {
	char *v_name;	/* Name of variable */
	char *v_mem;	/* Memory location	*/
	int v_type;	/* Variable type	*/
};

/* Variable table. The order of this table controls the
 * output order of the 'display' command
 */
struct	var_desc var_tab[] = {
	{	"Cref",		(char *)(&cref),	T_DOUBLE},
	{	"L0",		(char *)(&l0),		T_DOUBLE},
	{	"C0",		(char *)(&c0),		T_DOUBLE},
	{	"Lx0",		(char *)(&lx0),		T_DOUBLE},
	{	"Cx0",		(char *)(&cx0),		T_DOUBLE},
	{	"debug",	(char *)(&debug),	T_INT},
	{	NULL,		NULL,		NULL		},
};

/* Set/display variable */
int
doset (int argc,char *argv[],void *p)
{
	register struct var_desc *vp;
	int found = 0;
	int i,len;
	double dd;
  
	/* Nead at least 1 argument */
        if (argc <= 1) {
		puts("SET - Don't know what to do?\n");
		return -1;
	}

	if (strcmp(argv[1],"?") == 0) {
		for(vp = var_tab;vp->v_name != NULL;vp++){
			printf("%s\n",vp->v_name);
		}
		return 0;
	}
	for(vp = var_tab;vp->v_name != NULL;vp++){
		if(strncmpi(argv[1],vp->v_name,strlen(argv[1])) == 0){
			found = 1;
			break;
		}
	}
	if(!found){
		printf("unknown variable %s\n",argv[1]);
		return -1;
        }

	if (argc < 3) {
		/* Display variable */
		switch (vp->v_type) {
		case T_INT:
			printf("%s %d\n",vp->v_name,*(int *)vp->v_mem);
			break;
		case T_LONG:
			printf("%s %ld\n",vp->v_name,*(long *)vp->v_mem);
			break;
		case T_DOUBLE:
			printf("%s %g\n",vp->v_name,*(double *)vp->v_mem);
			break;
		case T_ON_OFF:
			printf("%s %s\n",vp->v_name,
				*(int *)vp->v_mem ? "on" : "off");
			break;
		case T_STRING:
			printf("%-13s : %s\n",
					vp->v_name,*(char **)vp->v_mem);
			break;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}
	} else {
		/* Set variable */
		switch (vp->v_type) {
		case T_INT:
			sscanf(argv[2],"%d",(int *)vp->v_mem);
			break;
		case T_LONG:
			sscanf(argv[2],"%ld",(long *)vp->v_mem);
			break;
		case T_DOUBLE:
			dd = atof(argv[2]);
			*(double *)vp->v_mem = dd;
			break;
		case T_ON_OFF:
			/* Hack (more or less ..) */
			if (strcmp(argv[2],"on") == 0)
				*(int *)vp->v_mem = 1;
			else
				*(int *)vp->v_mem = 0;
			break;
		case T_STRING:
			*(char **)vp->v_mem = strdup(argv[2]);
			break;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}
	}

	return 0;
}

#if	1
int
dodisplay (int argc,char *argv[],void *p)
{
	printf("Cref = ");print_c(cref);printf("\n");
	printf("C0   = ");print_c(c0);printf("\n");
	printf("L0   = ");print_l(l0);printf("\n");
	printf("Cx0  = ");print_c(cx0);printf("\n");
	printf("Lx0  = ");print_l(lx0);printf("\n");
	return(0);
}
#else
int
dodisplay (int argc,char *argv[],void *p)
{
	struct var_desc *vp;
	int cnt = 0;

	/* Display non-string variables */
	vp = var_tab;
	while (vp->v_name) {
		switch (vp->v_type) {
		case T_INT:
			printf("%-10s : %-6d    ",vp->v_name,*(int *)vp->v_mem);
			break;
		case T_LONG:
			printf("%-10s : %-6ld    ",
					vp->v_name,*(long *)vp->v_mem);
			break;
		case T_DOUBLE:
			printf("%-10s : %-6g    ",
					vp->v_name,*(double *)vp->v_mem);
			break;
		case T_ON_OFF:
			printf("%-10s : %-6s    ",vp->v_name,
				*(int *)vp->v_mem ? "on" : "off");
			break;
		case T_STRING:
			vp++;
			continue;
		default:
			printf("%s: unknown type %d\n",vp->v_name,vp->v_type);
		}
		if (++cnt == 3) {
			printf("\n");
			cnt = 0;
		}
		vp++;
	}
	if (cnt != 0)
		printf("\n");

	/* Display string variables */
	vp = var_tab;
	while (vp->v_name) {
		switch (vp->v_type) {
		case T_STRING:
			printf("%-13s : %s\n",
					vp->v_name,*(char **)vp->v_mem);
			break;
		default:
			vp++;
			continue;
		}
		vp++;
	}
	return 0;
}
#endif

